from pydantic import (
    BaseModel,
    StrictStr,
    StrictInt,
)
from .athena_encryption_configuration import AthenaEncryptionConfiguration


class AthenaSessionConfiguration(BaseModel):
    ExecutionRole: StrictStr
    WorkingDirectory: StrictStr
    IdleTimeoutSeconds: StrictInt
    EncryptionConfiguration: AthenaEncryptionConfiguration


from pydantic import (
    BaseModel,
    StrictStr,
)
from .athena_query_execution_context import AthenaQueryExecutionContext
from .athena_result_configuration import AthenaResultConfigruation
from .athena_result_reuse_configuration import AthenaResultReuseConfiguration


class AthenaStartQueryExecutionRequest(BaseModel):
    QueryString: StrictStr
    ClientRequestToken: StrictStr | None = None
    QueryExecutionContext: AthenaQueryExecutionContext | None = None
    ResultConfiguration: AthenaResultConfigruation | None = None
    WorkGroup: StrictStr | None = None
    ExecutionParameters: list[StrictStr] | None = None
    ResultReuseConfiguration: AthenaResultReuseConfiguration | None = None



from typing import Literal

AWSS3Acl = Literal[
    'private',
    'public-read',
    'public-read-write',
    'authenticated-read',
    'aws-exec-read',
    'bucket-owner-read',
    'bucket-owner-full-control'
]


from pydantic import (
    BaseModel,
    StrictStr,
)
from .athena_engine_configuration import AthenaEngineConfiguration
from .athena_session_configuration import AthenaSessionConfiguration
from .athena_session_status import AthenaSessionStatus
from .athena_session_statistics import AthenaSessionStatistics


class AthenaGetSessionResponse(BaseModel):
    SessionId: StrictStr
    Description: StrictStr | None = None
    WorkGroup: StrictStr | None = None
    EngineVersion: StrictStr
    EngineConfiguration: AthenaEngineConfiguration
    NotebookVersion: StrictStr
    SessionConfiguration: AthenaSessionConfiguration
    Status: AthenaSessionStatus
    Statistics: AthenaSessionStatistics



from pydantic import (
    BaseModel,
    StrictStr,
)
from .athena_session import AthenaSession


class AthenaListSessionsResponse(BaseModel):
    NextToken: StrictStr
    Sessions: list[AthenaSession]


from pydantic import (
    BaseModel,
    StrictStr,
    StrictInt,
    StrictBool,
)
from .athena_column_nullable import AthenaColumnNullable


class AthenaColumnInfo(BaseModel):
    CatalogName: StrictStr
    SchemaName: StrictStr
    TableName: StrictStr
    Label: StrictStr
    Type: StrictStr
    Precision: StrictInt
    Scale: StrictInt
    Nullable: AthenaColumnNullable
    CaseSensitive: StrictBool = False


from pydantic import (
    BaseModel,
    StrictStr
)
from .athena_connection_type import AthenaConnectionType
from .athena_data_catalog_status import AthenaDataCatalogStatus
from .athena_data_catalog_type import AthenaDataCatalogType


class AthenaDataCatalog(BaseModel):
    Name: StrictStr
    Description: StrictStr | None = None
    Type: AthenaDataCatalogType
    Parameters: dict[StrictStr, StrictStr] | None = None
    Status: AthenaDataCatalogStatus
    ConnectionType: AthenaConnectionType
    Error: StrictStr | None = None


from abc import ABC, abstractmethod
from botocore.paginate import Paginator
from typing import Literal, Any


AthenaQueryResultType = Literal['DATA_MANIFEST', 'DATA_ROWS']

AthenaDataCatalogType = Literal[
    'LAMBDA',
    'GLUE',
    'HIVE',
    'FEDERATED',
]

AthenaClientTag = dict[
    Literal['Key', 'Value'],
    str,
]

AthenaCalculationExecution = dict[
    Literal["CodeBlock"],
    str,
]

AthenaResultReuseConfiguration = dict[
    Literal["ResultReuseByAgeConfiguration"],
    dict[
        Literal["Enabled", "MaxAgeInMinutes"],
        bool | int
    ]
]

AthenaQueryExecutionContext = dict[
    Literal["Database", "Catalog"],
    str,
]

AthenaEncryptionOption = Literal['SSE_S3','SSE_KMS','CSE_KMS']

AthenaAclConfiguration = dict[
    Literal["S3AclOption"],
    str
]

AthenaEncryptionConfiguration = dict[
    Literal["EncryptionOption", "KmsKey"],
    AthenaEncryptionOption | str
]

AthenaResultConfiguration = dict[
    Literal[
        "OutputLocation",
        "EncryptionConfiguration",
        "ExpectedBucketOwner",
        "AclConfiguration",
    ],
    str | AthenaAclConfiguration | AthenaEncryptionConfiguration
]

AthenaSessionStateFilter = Literal[
    'CREATING',
    'CREATED',
    'IDLE',
    'BUSY',
    'TERMINATING',
    'TERMINATED',
    'DEGRADED',
    'FAILED',
]

AthenaEngineConfiguration = dict[
    Literal[
        "CoordinatorDpuSize",
        "MaxConcurrentDpus",
        "DefaultExecutorDpuSize",
        "AdditionalConfigs",
        "SparkProperties",
    ],
    int | dict[str, str]
]

class AthenaClient(ABC):

    @abstractmethod
    def create_data_catalog(
        self,
        Name: str = None,
        Type: AthenaDataCatalogType = None,
        Description: str = None,
        Parameters: dict[str, str] | None = None,
        Tags: list[AthenaClientTag] | None = None,
    ) -> dict[str, Any]:
        pass

    @abstractmethod
    def create_named_query(
        self,
        Name: str = None,
        Description: str = None,
        Database: str = None,
        QueryString: str = None,
        ClientRequestToken: str = None,
        WorkGroup: str = None,
    ) -> dict[str, Any]:
        pass

    @abstractmethod
    def create_prepared_statement(
        self,
        StatementName: str = None,
        WorkGroup: str = None,
        QueryStatement: str = None,
        Description: str = None,
    ) -> dict[str, Any]:
        pass

    @abstractmethod
    def get_calculation_execution(
        self,
        CalculationExecutionId: str = None,
    ) -> dict[str, Any]:
        pass

    @abstractmethod
    def get_calculation_execution_code(
        self,
        CalculationExecutionId: str = None,
    ) -> dict[str, Any]:
        pass

    @abstractmethod
    def get_calculation_execution_status(
        self,
        CalculationExecutionId: str = None,

    ) -> dict[str, Any]:
        pass

    @abstractmethod
    def get_data_catalog(
        self,
        Name: str = None,
        WorkGroup: str = None,
    ) -> dict[str, Any]:
        pass

    @abstractmethod
    def get_database(
        self,
        CatalogName: str = None,
        DatabaseName: str = None,
        WorkGroup: str = None,
    ) -> dict[str, Any]:
        pass

    @abstractmethod
    def get_named_query(
        self,
        NamedQueryId: str = None
    ) -> dict[str, Any]:
        pass

    @abstractmethod
    def get_prepared_statement(
        self,
        StatementName: str = None,
        WorkGroup: str = None
    ) -> dict[str, Any]:
        pass

    @abstractmethod
    def get_query_execution(
        self,
        QueryExecutionId: str = None,
    ) -> dict[str, Any]:
        pass

    @abstractmethod
    def get_query_results(
        self,
        QueryExecutionId: str = None,
        NextToken: str = None,
        MaxResults: int = None,
        QueryResultType: AthenaQueryResultType = None,
    ) -> dict[str, Any]:
        pass

    @abstractmethod
    def list_named_queries(
        self,
        NextToken: str | None = None,
        MaxResults: int | None = None,
        WorkGroup: str | None = None
    ) -> list[dict[str, Any]]:
        pass

    @abstractmethod
    def list_query_executions(
        self,
        NextToken: str | None = None,
        MaxResults: int | None = None,
        WorkGroup: str | None = None
    ) -> list[dict[str, Any]]:
        pass

    @abstractmethod
    def list_sessions(
        self,
        WorkGroup='string',
        StateFilter: AthenaSessionStateFilter = None,
        MaxResults=123,
        NextToken='string'
    ) -> list[dict[str, Any]]:
        pass

    @abstractmethod
    def list_prepared_statements(
        self,
        NextToken: str | None = None,
        MaxResults: int | None = None,
        WorkGroup: str | None = None
    ) -> list[dict[str, Any]]:
        pass

    @abstractmethod
    def list_databases(
        self,
        CatalogName: str = None,
        NextToken: str = None,
        MaxResults: int = None,
        WorkGroup: str = None
    ) -> list[dict[str, Any]]:
        pass

    @abstractmethod
    def list_data_catalogs(
        self,
        NextToken: str = None,
        MaxResults: int = None,
        WorkGroup: str = None,
    ) -> list[dict[str, Any]]:
        pass

    @abstractmethod
    def get_session(
        self,
        SessionId: str = None
    ) -> dict[str, Any]:
        pass

    @abstractmethod
    def get_session_status(
        self,
        SessionId: str = None
    ) -> dict[str, Any]:
        pass

    @abstractmethod
    def start_calculation_execution(
        self,
        SessionId: str = None,
        Description: str = None,
        CalculationConfiguration: AthenaCalculationExecution = None,
        CodeBlock: str = None,
        ClientRequestToken: str = None
    ) -> dict[str, Any]:
        pass

    @abstractmethod
    def start_query_execution(
        self,
        QueryString: str = None,
        ClientRequestToken: str = None,
        QueryExecutionContext: AthenaQueryExecutionContext = None,
        ResultConfiguration: AthenaResultConfiguration = None,
        WorkGroup: str = None,
        ExecutionParameters: list[str] = None,
        ResultReuseConfiguration: AthenaResultReuseConfiguration = None
    ) -> dict[str, Any]:
        pass

    @abstractmethod
    def start_session(
        self,
        Description: str = None,
        WorkGroup: str = None,
        EngineConfiguration: AthenaEngineConfiguration = None,
        NotebookVersion: str = None,
        SessionIdleTimeoutInMinutes: int = None,
        ClientRequestToken: str = None
    ) -> dict[str, Any]:
        pass

    @abstractmethod
    def stop_calculation_execution(
        self,
        CalculationExecutionId: str = None
    ) -> dict[str, Any]:
        pass

    @abstractmethod
    def stop_query_execution(
        self,
        QueryExecutionId: str = None
    ) -> dict[str, Any]:
        pass

    @abstractmethod
    def terminate_session(
        self,
        SessionId: str = None
    ) -> dict[str, Any]:
        pass

    @abstractmethod
    def update_data_catalog(
        self,
        Name: str = None,
        Type: AthenaDataCatalogType = None,
        Description: str = None,
        Parameters: dict[str, str] = None
    ) -> dict[str, Any]:
        pass

    @abstractmethod
    def update_named_query(
        self,
        NamedQueryId: str = None,
        Name: str = None,
        Description: str = None,
        QueryString: str = None,
    ) -> dict[str, Any]:
        pass

    @abstractmethod
    def delete_named_query(
        self,
        NamedQueryId: str = None
    ) -> dict[str, Any]:
        pass

    @abstractmethod
    def delete_prepared_statement(
        self,
        StatementName: str = None,
        WorkGroup: str = None,
    ) -> dict[str, Any]:
        pass

    @abstractmethod
    def delete_data_catalog(
        self,
        Name: str = None,
        DeleteCatalogOnly: bool = False
    ) -> dict[str, Any]:
        pass

    @abstractmethod
    def get_paginator(
        self,
        Operation: str,
    ) -> Paginator:
        pass

    @abstractmethod
    def close(self):
        pass


from pydantic import (
    BaseModel,
    StrictInt,
    StrictStr,
)
from .athena_result_reuse_information import AthenaResultReuseInformation


class AthenaQueryStatistics(BaseModel):
    EngineExecutionTimeInMillis: StrictInt
    DataScannedInBytes: StrictInt
    DataManifestLocation: StrictStr
    TotalExecutionTimeInMillis: StrictInt
    QueryQueueTimeInMillis: StrictInt
    ServicePreProcessingTimeInMillis: StrictInt
    QueryPlanningTimeInMillis: StrictInt
    ServiceProcessingTimeInMillis: StrictInt
    ResultReuseInformation: AthenaResultReuseInformation


from pydantic import (
    BaseModel,
    StrictStr,
)
from .athena_calculation_configuration import AthenaCalculationConfiguration


class AthenaStartCalculationExecutionRequest(BaseModel):
    SessionId: StrictStr
    Description: StrictStr | None = None
    CalculationConfigruation: AthenaCalculationConfiguration | None = None
    CodeBlock: StrictStr | None = None
    ClientRequesttoken: StrictStr | None = None


from pydantic import (
    StrictStr
)
from typing import Optional
from werkflow_aws.models.base import AWSBoto3Base


class AWSs3GetBucketTaggingOptions(AWSBoto3Base):
    expected_bucket_owner: Optional[StrictStr]=None


from pydantic import (
    BaseModel,
    StrictBool,
    StrictStr,
)


class AthenaQueryResultsS3AccessGrantsConfiguration(BaseModel):
    EnableS3AccessGrants: StrictBool = False
    CreateUserLevelPrefix: StrictBool = False
    AuthenticationType: StrictStr


from pydantic import (
    BaseModel,
    StrictStr,
)
from .athena_engine_version import AthenaEngineVersion
from .athena_session_status import AthenaSessionStatus


class AthenaSession(BaseModel):
    SessionId: StrictStr
    Description: StrictStr | None = None
    EngineVersion: AthenaEngineVersion
    NotebookVersion: StrictStr
    Status: AthenaSessionStatus



from pydantic import BaseModel, StrictStr


class AWSSecretsManagerGetSecretValueRequest(BaseModel):
    SecretId: StrictStr
    VersionId: StrictStr | None = None
    VersionStage: StrictStr | None = None



import datetime
from pydantic import (
    BaseModel,
    StrictStr,
    StrictInt,
    StrictBool,
    StrictBytes
)
from typing import Dict, BinaryIO, TextIO
from .aws_s3_checksum_algorithm import AWSS3ChecksumAlgorithm
from .aws_s3_acl import AWSS3Acl
from .aws_s3_server_side_encryption import AWSS3ServerSideEncryption
from .aws_s3_storage_class import AWSS3StorageClass
from .aws_s3_object_lock_mode import AWSS3ObjectLockMode
from .aws_s3_object_lock_legal_hold_status import AWSS3ObjectLockLegalHoldStatus


class AWSS3PutObjectRequest(BaseModel):
    Bucket: StrictStr
    Key: StrictStr
    Body: StrictBytes | BinaryIO | TextIO
    ACL: AWSS3Acl | None=None
    CacheControl: StrictStr | None = None
    ContentDisposition: StrictStr | None =None
    ContentEncoding: StrictStr | None =None
    ContentLanguage: StrictStr | None=None
    ContentLength: StrictInt | None = None
    ContentMD5: StrictStr | None = None
    ContentType: StrictStr | None = None
    ChecksumAlgorithm: AWSS3ChecksumAlgorithm | None =None
    ChecksumCRC32: StrictStr | None = None
    ChecksumCRC32C: StrictStr | None = None
    ChecksumSHA1: StrictStr | None = None
    ChecksumSHA256: StrictStr | None = None
    Expires: datetime.datetime | None = None
    GrantFullControl: StrictStr | None = None
    GrantRead: StrictStr | None = None
    GrantReadACP: StrictStr | None = None
    GrantWriteACP: StrictStr | None = None
    Metadata: Dict[str, str] | None =None
    server_side_encryption: AWSS3ServerSideEncryption | None = None
    storage_class: AWSS3StorageClass | None =None
    WebsiteRedirectionLocation: StrictStr | None = None
    SSECustomerAlgorithm: StrictStr | None = None
    SSECustomerKey: StrictStr | None = None
    SSEKMSKeyId: StrictStr | None = None
    SSEKMSEncryptionContext: StrictStr | None = None
    BucketKeyEnabled: StrictBool | None = None
    Tagging: StrictStr | None = None
    ObjectLockMode: AWSS3ObjectLockMode | None = None
    ObjectLockRetainUntilDate: datetime.datetime | None = None
    ObjectLockLegalHoldStatus: AWSS3ObjectLockLegalHoldStatus | None = None
    ExpectedbucketOwner: StrictStr | None = None



from pydantic import (
    BaseModel,
    StrictStr,
    StrictInt
)
from .athena_engine_configuration import AthenaEngineConfiguration


class AthenaStartSessionRequest(BaseModel):
    Description: StrictStr | None = None
    WorkGroup: StrictStr
    EngineConfiguration: AthenaEngineConfiguration
    NotebookVersion: StrictStr | None = None
    SessionIdleTimeoutInMinutes: StrictInt | None = None
    ClientRequestToken: StrictStr | None = None
    


from __future__ import annotations
from abc import ABC, abstractmethod
from typing import (
    Any
)


class SecretsManagerClient(ABC):

    @abstractmethod
    def get_secret_value(
        self,
        SecretId: str = None,
        VersionId: str | None = None,
        VersionStage: str | None = None,
    ) -> dict[str, Any]:
        pass

    @abstractmethod
    def close(self):
        pass


from pydantic import (
    BaseModel,
    StrictStr,
)


class AthenaDatabase(BaseModel):
    Name: StrictStr
    Description: StrictStr | None = None
    Parameters: dict[StrictStr, StrictStr] | None = None


from pydantic import (
    BaseModel,
    StrictStr,
)
from .athena_engine_version import AthenaEngineVersion
from .athena_managed_query_results_configuration import AthenaManagedQueryResultsConfiguration
from .athena_statement_type import AthenaStatementType
from .athena_result_configuration import AthenaResultConfigruation
from .athena_result_reuse_configuration import AthenaResultReuseConfiguration
from .athena_query_execution_context import AthenaQueryExecutionContext
from .athena_query_status import AthenaQueryStatus
from .athena_query_statistics import AthenaQueryStatistics
from .athena_query_results_s3_access_grants_configruation import AthenaQueryResultsS3AccessGrantsConfiguration


class AthenaQueryExecution(BaseModel):
    QueryExecutionId: StrictStr
    Query: StrictStr
    StatementType: AthenaStatementType
    ManagedQueryResultsConfiguration: AthenaManagedQueryResultsConfiguration
    ResultConfiguration: AthenaResultConfigruation
    ResultReuseConfiguration: AthenaResultReuseConfiguration
    QueryExecutionContext: AthenaQueryExecutionContext
    Status: AthenaQueryStatus
    Statistics: AthenaQueryStatistics
    WorkGroup: StrictStr
    EngineVersion: AthenaEngineVersion
    ExecutionParameters: list[StrictStr]
    SubstatementType: StrictStr
    QueryResultsS3AccessGrantsConfiguration: AthenaQueryResultsS3AccessGrantsConfiguration


from pydantic import (
    BaseModel,
    StrictStr,
)
from .athena_data_catalog_type import AthenaDataCatalogType


class AthenaUpdateDataCatalogRequest(BaseModel):
    Name: StrictStr
    Type: AthenaDataCatalogType
    Description: StrictStr | None = None
    Parameters: dict[StrictStr, StrictStr] | None = None


from pydantic import (
    BaseModel,
    StrictStr,
)
from .athena_encryption_configuration import AthenaEncryptionConfiguration
from .athena_acl_configuration import AthenaAclConfiguration


class AthenaResultConfigruation(BaseModel):
    OutputLocation: StrictStr
    EncryptionConfiguration: AthenaEncryptionConfiguration
    ExpectedBucketOwner: StrictStr
    AclConfiguration: AthenaAclConfiguration



from pydantic import (
    BaseModel,
    StrictInt,
    StrictStr,
)
from .athena_result_set import AthenaResultSet


class AthenaGetQueryResultsResponse(BaseModel):
    UpdateCount: StrictInt
    ResultSet: AthenaResultSet
    NextToken: StrictStr


from .aws_secrets_manager_get_secret_value_request import AWSSecretsManagerGetSecretValueRequest as AWSSecretsManagerGetSecretValueRequest
from .aws_secrets_manager_get_secret_value_response import AWSSecretManagerGetSecretValueResponse as AWSSecretManagerGetSecretValueResponse


from pydantic import (
    BaseModel,
    StrictStr,
    StrictBool
)


class AWSS3DeleteObjectRequest(BaseModel):
    Bucket: StrictStr
    Key: StrictStr
    MFA: StrictStr | None = None
    VersionId: StrictStr | None = None
    BypassGovernanceRetention: StrictBool | None = None
    ExpectedBucketOwner: StrictStr | None = None




from pydantic import (
    StrictStr,
    StrictInt
)

from werkflow_aws.models.base import AWSBoto3Base


class AWSs3ListMultipartUploadsRequest(AWSBoto3Base):
    Bucket: StrictStr
    Delimiter: StrictStr | None=None
    EncodingType: StrictStr | None=None
    KeyMarker: StrictStr | None=None
    MaxUploads: StrictInt | None=None
    Prefix: StrictStr | None=None
    UploadIdMarker: StrictStr | None=None
    ExpectedBucketOwner: StrictStr | None=None
    RequestPayer: StrictStr | None = None



from pydantic import (
    BaseModel,
    StrictStr,
)
from .athena_data_catalog_type import AthenaDataCatalogType
from .athena_data_catalog_status import AthenaDataCatalogStatus
from .athena_connection_type import AthenaConnectionType


class AthenaDataCatalogSummary(BaseModel):
    CatalogName: StrictStr
    Type: AthenaDataCatalogType
    Status: AthenaDataCatalogStatus
    ConnectionType: AthenaConnectionType
    Error: StrictStr | None = None


from pydantic import (
    BaseModel,
    StrictStr,
)
from .athena_encryption_option import AthenaEncryptionOption


class AthenaEncryptionConfiguration(BaseModel):
    KmsKey: StrictStr
    EncryptionOption: AthenaEncryptionOption


from pydantic import BaseModel, Field
from .aws_s3_multipart_upload_part import AWSs3MultipartUploadPart

class AWSs3MultipartUpload(BaseModel):
    Parts: list[AWSs3MultipartUploadPart] = Field(min_length=1)



from pydantic import (
    BaseModel,
    StrictStr,
    StrictInt
)


class AWSs3MultipartUploadPart(BaseModel):
    ChecksumCRC32: StrictStr | None=None
    ChecksumCRC32C: StrictStr | None=None
    ChecksumCRC64NVME: StrictStr | None = None
    ChecksumSHA1: StrictStr | None=None
    ChecksumSHA256: StrictStr | None=None
    ETag: StrictStr
    PartNumber: StrictInt



from pydantic import BaseModel
from .athena_result_set import AthenaResultSet
from .athena_result_set_metadata import AthenaResultSetMetadata


class AthenaResultSet(BaseModel):
    Rows: list[AthenaResultSet]
    ResultSetMetadata: AthenaResultSetMetadata


from pydantic import BaseModel, StrictStr, StrictInt
from typing import Literal
from .aws_s3_multipart_upload import AWSs3MultipartUpload


class AWSs3CompleteMultipartUploadRequest(BaseModel):
    Bucket: StrictStr
    Key: StrictStr
    MultipartUpload: AWSs3MultipartUpload
    UploadId: StrictStr
    ChecksumCRC32: StrictStr | None = None
    ChecksumCRC32C: StrictStr | None = None
    ChecksumCRC64NVME: StrictStr | None = None
    ChecksumSHA1: StrictStr | None = None
    ChecksumSHA256: StrictStr | None = None
    ChecksumType: Literal["COMPOSITE", "FULL_OBJECT"] | None = None
    MpuObjectSize: StrictInt | None = None
    RequestPayer: Literal["requester"] | None = None
    ExpectedBucketOwner: StrictStr | None = None
    IfMatch: StrictStr | None = None
    IfNoneMatch: StrictStr | None = None
    SSECustomerAlgorithm: StrictStr | None = None
    SSECustomerKey: StrictStr | None = None



from pydantic import (
    BaseModel,
    StrictStr,
    StrictInt,
)
from .athena_session_state import AthenaSessionState


class AthenaListSessionsRequest(BaseModel):
    WorkGroup: StrictStr
    StateFilter: AthenaSessionState | None = None
    MaxResults: StrictInt | None = None
    NextToken: StrictStr | None = None


from pydantic import (
    BaseModel,
    StrictStr,
)
from .athena_database import AthenaDatabase


class AthenaListDatabasesResponse(BaseModel):
    DatabaseList: list[AthenaDatabase]
    NextToken: StrictStr | None = None


from pydantic import (
    BaseModel,
    StrictStr,
)
from .athena_calculation_state import AthenaCalculationState


class AthenaStartCalculationExecutionResponse(BaseModel):
    CalculationExecutionId: StrictStr
    State: AthenaCalculationState


import io
from pydantic import (
    BaseModel,
    StrictStr,
    StrictInt,
    StrictBytes,
    ConfigDict
)
from typing import Literal


class AWSs3UploadPartRequest(BaseModel):
    Body: StrictBytes | io.FileIO | io.BytesIO | io.StringIO
    Bucket: StrictStr
    Key: StrictStr
    PartNumber: StrictInt
    UploadId: StrictStr
    ContentLength: StrictInt | None = None
    ContentMD5: StrictStr | None = None
    ChecksumAlgorithm: Literal[
        'CRC32',
        'CRC32C',
        'SHA1',
        'SHA256'
    ] | None = None
    ChecksumCRC32: StrictStr | None = None
    ChecksumCRC32C: StrictStr | None = None
    ChecksumCRC64NVME: StrictStr | None = None
    ChecksumSHA1: StrictStr | None = None
    ChecksumSHA256: StrictStr | None = None
    SSECustomerAlgorithm: StrictStr | None = None
    SSECustomerKey: StrictStr | None = None
    RequestPayer: Literal['requester'] | None = None
    ExpectedBucketOwner: StrictStr | None = None

    model_config = ConfigDict(arbitrary_types_allowed=True)



import datetime
from pydantic import BaseModel, StrictStr, StrictBytes


class AWSSecretManagerGetSecretValueResponse(BaseModel):
    ARN: StrictStr
    Name: StrictStr
    VersionId: StrictStr | None = None
    SecretBinary: StrictBytes
    SecretString: StrictStr
    VersionStages: list[StrictStr] | None = None
    CreatedDate: datetime.datetime



from typing import Literal


AthenaConnectionType = Literal[
    'DYNAMODB',
    'MYSQL',
    'POSTGRESQL',
    'REDSHIFT',
    'ORACLE',
    'SYNAPSE',
    'SQLSERVER',
    'DB2',
    'OPENSEARCH',
    'BIGQUERY',
    'GOOGLECLOUDSTORAGE',
    'HBASE',
    'DOCUMENTDB',
    'CMDB',
    'TPCDS',
    'TIMESTREAM',
    'SAPHANA',
    'SNOWFLAKE',
    'DATALAKEGEN2',
    'DB2AS400',
]




from pydantic import (
    BaseModel,
    StrictStr,
)


class AthenaCreateNamedQueryRequest(BaseModel):
    Name: StrictStr
    Description: StrictStr | None = None
    Database: StrictStr
    QueryString: StrictStr
    ClientRequestToken: StrictStr | None = None
    WorkGroup: StrictStr | None = None


from pydantic import (
    BaseModel,
    StrictStr,
)
from .athena_session_state import AthenaSessionState


class AthenaStartSessionResponse(BaseModel):
    SessionId: StrictStr
    State: AthenaSessionState


from pydantic import (
    BaseModel,
    StrictBool,
)
from .athena_encryption_configuration import AthenaEncryptionConfiguration


class AthenaManagedQueryResultsConfiguration(BaseModel):
    Enabled: StrictBool = True
    EncryptionConfiguration: AthenaEncryptionConfiguration


from typing import Literal


SupportedClients = Literal[
    "athena",
    "cloudwatch",
    "code_artifact",
    "cost_explorer",
    "ec2",
    "elasticache",
    "msk",
    "organizations",
    "sts",
]



from pydantic import (
    BaseModel,
    StrictStr,
    StrictInt,
)


class AthenaListDatabasesRequest(BaseModel):
    CatalogName: StrictStr
    NextToken: StrictStr | None = None
    MaxResults: StrictInt | None = None
    WorkGroup: StrictStr | None = None


import datetime
from pydantic import (
    BaseModel,
    StrictStr,
    StrictInt,
)

from .aws_s3_checksum_mode import AWSS3ChecksumMode


class AWSS3GetObjectRequest(BaseModel):
    Bucket: StrictStr
    Key: StrictStr
    IfMatch: StrictStr | None = None
    IfModifiedSince: datetime.datetime | None = None
    IfNoneMatch: StrictStr | None = None
    IfUnmodifiedSince: datetime.datetime | None = None
    Range: StrictStr | None = None
    ResponseCacheControl: StrictStr | None = None
    ResponseContentDisposition: StrictStr | None = None
    ResponseContentEncoding: StrictStr | None = None
    ResponseContentLanguage: StrictStr | None = None
    ResponseContentType: StrictStr | None = None
    ResponseExpires: datetime.datetime | None = None
    VersionId: StrictStr | None = None
    SSECustomerAlgorithm: StrictStr | None = None
    SSECustomerKey: StrictStr | None = None
    PartNumber: StrictInt | None = None
    ExpectedBucketOwner: StrictStr | None = None
    ChecksumMode: AWSS3ChecksumMode | None = None


from pydantic import (
    BaseModel,
    StrictStr,
)

class AthenaCreatePreparedStatementRequest(BaseModel):
    StatementName: StrictStr
    WorkGroup: StrictStr
    QueryStatement: StrictStr
    Description: StrictStr | None = None


from werkflow_core import Module
from .parsers import AWSConfigParser
from .services import (
    AWSAthena,
    AWSCloudwatch,
    AWSCredentials,
    AWSCodeArtifact,
    AWSCostExplorer,
    AWSEC2,
    AWSElastiCache,
    AWSMSK,
    AWSOrganizations,
    AWSSTS,
)
from werkflow_aws.models import (
    AWSCredentialsSet,
    RegionName,
)
from werkflow_aws.models.sts import AssumeRoleRequest
from .authorization_type import AuthorizationType
from .supported_clients import SupportedClients

class AWS(Module):

    def __init__(self) -> None:
        super().__init__()

        self.athena = AWSAthena()
        self.cloudwatch = AWSCloudwatch()
        self.config = AWSConfigParser()
        self.credentials = AWSCredentials()
        self.code_artifact = AWSCodeArtifact()
        self.cost_explorer = AWSCostExplorer()
        self.ec2 = AWSEC2()
        self.elasticache = AWSElastiCache()
        self.msk = AWSMSK()
        self.organizations = AWSOrganizations()
        self.sts = AWSSTS()

    async def connect(
        self,
        client: SupportedClients,
        credentials: AWSCredentialsSet, 
        region: RegionName,
        authorization_type: AuthorizationType = "client"
    ):
        match authorization_type:
            case "assume":
                pass
            case "sso":
                pass
            case _:
                await self._connect_client(
                    client,
                    credentials,
                    region,
                )
    
    async def _connect_assumed_role(
        self,
        client: SupportedClients,
        assume_role_request: AssumeRoleRequest,
        assume_proxy_role_request: AssumeRoleRequest,
        region: RegionName,
    ):
        response = await self.sts.assume_role_by_proxy(
            assume_role_request,
            assume_proxy_role_request,
            region,
        )

        await self._connect_client(
            client,
            AWSCredentialsSet(
                aws_access_key_id=response.Credentials.AccessKeyId,
                aws_secret_access_key=response.Credentials.SecretAccessKey,
                aws_session_token=response.Credentials.SessionToken,
            ),
            region,
        )

    async def _connect_sso(
        self,
        client: SupportedClients,
        credentials: AWSCredentialsSet, 
    ):
        match client:
            case "athena":
                await self.athena.sso(credentials.aws_profile)
            
            case "cloudwatch":
                await self.cloudwatch.sso(credentials.aws_profile)

            case "code_artifact":
                await self.code_artifact.sso(credentials.aws_profile)
            
            case "ec2":
                await self.code_artifact.sso(credentials.aws_profile)
            
            case "elasticache":
                await self.elasticache.sso(credentials.aws_profile)

            case "msk":
                await self.msk.sso(credentials.aws_profile)

            case "organizations":
                await self.organizations.sso(credentials.aws_profile)

            case "sts": 
                await self.organizations.sso(credentials.aws_profile)
                
    
    async def _connect_client(
        self,
        client: SupportedClients,
        credentials: AWSCredentialsSet, 
        region: RegionName,
    ):
        match client:
            case "athena":
                await self.athena.connect(credentials, region)
            
            case "cloudwatch":
                await self.cloudwatch.connect(credentials, region)

            case "code_artifact":
                await self.code_artifact.connect(credentials, region)
            
            case "ec2":
                await self.code_artifact.connect(credentials, region)
            
            case "elasticache":
                await self.elasticache.connect(credentials, region)

            case "msk":
                await self.msk.connect(credentials, region)

            case "organizations":
                await self.organizations.connect(credentials, region)

            case "sts": 
                await self.organizations.connect(credentials, region)
                
    

    async def close(self):
        await self.athena.close()
        await self.cloudwatch.close()
        await self.credentials.close()
        await self.code_artifact.close()
        await self.cost_explorer.close()
        await self.ec2.close()
        await self.elasticache.close()
        await self.msk.close()
        await self.organizations.close()
        await self.sts.close()

    def abort(self):
        self.athena.abort()
        self.cloudwatch.abort()
        self.credentials.abort()
        self.code_artifact.abort()
        self.cost_explorer.abort()
        self.ec2.abort()
        self.elasticache.abort()
        self.msk.abort()
        self.organizations.abort()
        self.sts.abort()

    




from pydantic import (
    BaseModel,
    StrictStr,
)
from .athena_prepared_statement import AthenaPreparedStatement


class AthenaListPreparedStatementsResponse(BaseModel):
    PreparedStatements: list[AthenaPreparedStatement]
    NextToken: StrictStr | None = None


from pydantic import (
    BaseModel,
    StrictStr,
)


class AthenaUpdateNamedQueryRequest(BaseModel):
    NamedQueryId: StrictStr
    Name: StrictStr
    Description: StrictStr | None = None
    QueryString: StrictStr


import datetime
from pydantic import BaseModel, StrictStr, StrictInt
from typing import Literal, Dict


class AWSs3UploadPartCopyRequest(BaseModel):
    Bucket: StrictStr
    Key: StrictStr
    PartNumber: StrictInt
    UploadId: StrictStr
    CopySource:  StrictStr | Dict[
        Literal['bucket','key','version_id'],
        StrictStr,
    ] | None = None
    CopySourceIfMatch: StrictStr | None = None
    CopySourceIfModifiedSince: datetime.datetime | None = None
    CopySourceIfNoneMatch: StrictStr | None = None
    CopySourceIfUnmodifiedSince: datetime.datetime | None = None
    CopySourceRange: StrictStr | None = None
    SSECustomerAlgorithm: StrictStr | None = None
    SSECustomerKey: StrictStr | None = None
    CopySourceSSECustomerAlgorithm: StrictStr | None = None
    CopySourceSSECustomerKey: StrictStr | None = None
    RequestPayer: Literal['requester'] | None = None
    ExpectedBucketOwner: StrictStr | None = None
    ExpectedSourceBucketOwner: StrictStr | None = None



from pydantic import (
    BaseModel,
    StrictStr,
    StrictInt,
)
from .athena_query_result_type import AthenaQueryResultType


class AthenaGetQueryResultsRequest(BaseModel):
    QueryExecutionId: StrictStr
    NextToken: StrictStr | None = None
    MaxResults: StrictInt | None = None
    QueryResultType: AthenaQueryResultType | None = None


from typing import Literal


AthenaDataCatalogStatus = Literal[
    'CREATE_IN_PROGRESS',
    'CREATE_COMPLETE',
    'CREATE_FAILED',
    'CREATE_FAILED_CLEANUP_IN_PROGRESS',
    'CREATE_FAILED_CLEANUP_COMPLETE',
    'CREATE_FAILED_CLEANUP_FAILED',
    'DELETE_IN_PROGRESS',
    'DELETE_COMPLETE',
    'DELETE_FAILED',
]


from pydantic import BaseModel
from .athena_calculation_status import AthenaCalculationStatus
from .athena_calculation_statistics import AthenaCalculationStatistics


class AthenaGetCalculationExecutionStatusResponse(BaseModel):
    Status: AthenaCalculationStatus
    Statistics: AthenaCalculationStatistics


import datetime
from pydantic import (
    BaseModel,
    StrictStr,
)
from .athena_session_state import AthenaSessionState


class AthenaSessionStatus(BaseModel):
    StartDateTime: datetime.datetime
    LastModifiedDateTime: datetime.datetime
    EndDateTime: datetime.datetime
    IdleSinceDateTime: datetime.datetime
    State: AthenaSessionState
    StateChangeReason: StrictStr



from pydantic import (
    BaseModel,
    StrictStr,
)
from .athena_session_status import AthenaSessionStatus


class AthenaGetSessionStatusResponse(BaseModel):
    SessionId: StrictStr
    Status: AthenaSessionStatus


import datetime
from pydantic import (
    BaseModel,
    StrictStr,
)
from .athena_query_error import AthenaQueryError
from .athena_query_state import AthenaQueryState


class AthenaQueryStatus(BaseModel):
    State: AthenaQueryState
    StateChangeReason: StrictStr | None = None
    SubmissionDateTime: datetime.datetime
    CompletionDateTime: datetime.datetime
    AthenaError: AthenaQueryError


from pydantic import (
    BaseModel,
    StrictStr,
)
from .athena_calculation_status import AthenaCalculationStatus
from .athena_calculation_statistics import AthenaCalculationStatistics
from .athena_calculation_result import AthenaCalculationResult


class AthenaGetCalculationExecutionResponse(BaseModel):
    CalculationExecutionId: StrictStr
    SessionId: StrictStr
    Description: StrictStr | None = None
    WorkingDirection: StrictStr
    Status: AthenaCalculationStatus
    Statistics: AthenaCalculationStatistics
    Result: AthenaCalculationResult


import asyncio
import boto3
import functools
from botocore.config import Config
from concurrent.futures import ThreadPoolExecutor
from werkflow_aws.exceptions import UnsetAWSConnectionException
from werkflow_aws.models import (
    AWSCredentialsSet,
    AWSRegionMap,
    RegionName,
)
from werkflow_aws.models.athena import (
    AthenaCreateDataCatalogRequest,
    AthenaCreateDataCatalogResponse,
    AthenaCreateNamedQueryRequest,
    AthenaCreateNamedQueryResponse,
    AthenaCreatePreparedStatementRequest,
    AthenaCreatePreparedStatementResponse,
    AthenaGetCalculationExectutionCodeRequest,
    AthenaGetCalculationExecutionCodeResponse,
    AthenaGetCalculationExecutionRequest,
    AthenaGetCalculationExecutionResponse,
    AthenaGetCalculationExecutionStatusRequest,
    AthenaGetCalculationExecutionStatusResponse,
    AthenaGetDataCatalogRequest,
    AthenaGetDataCatalogResponse,
    AthenaGetDatabaseRequest,
    AthenaGetDatabaseResponse,
    AthenaGetNamedQueryRequest,
    AthenaGetNamedQueryResponse,
    AthenaGetPreparedStatementRequest,
    AthenaGetPreparedStatementResponse,
    AthenaGetQueryExecutionRequest,
    AthenaGetQueryExecutionResponse,
    AthenaGetQueryResultsRequest,
    AthenaGetQueryResultsResponse,
    AthenaGetSessionRequest,
    AthenaGetSessionResponse,
    AthenaGetSessionStatusRequest,
    AthenaGetSessionStatusResponse,
    AthenaListNamedQueriesRequest,
    AthenaListNamedQueriesResponse,
    AthenaListQueryExecutionsRequest,
    AthenaListQueryExecutionsResponse,
    AthenaListDatabasesRequest,
    AthenaListDatabasesResponse,
    AthenaListDataCatalogsRequest,
    AthenaListDataCatalogsResponse,
    AthenaListPreparedStatementsRequest,
    AthenaListPreparedStatementsResponse,
    AthenaListSessionsRequest,
    AthenaListSessionsResponse,
    AthenaStartCalculationExecutionRequest,
    AthenaStartCalculationExecutionResponse,
    AthenaStartQueryExecutionRequest,
    AthenaStartQueryExecutionResponse,
    AthenaStartSessionRequest,
    AthenaStartSessionResponse,
    AthenaStopCalculationExecutionRequest,
    AthenaStopCalculationExecutionResponse,
    AthenaStopQueryExecutionRequest,
    AthenaStopQueryExecutionResponse,
    AthenaTerminateSessionRequest,
    AthenaTerminateSessionResponse,
    AthenaUpdateDataCatalogRequest,
    AthenaUpdateDataCatalogResponse,
    AthenaUpdateNamedQueryRequest,
    AthenaUpdateNamedQueryResponse,
)

from werkflow_aws.helpers import collect_paginator
from werkflow_aws.types import AthenaClient

from werkflow_system import System


class AWSAthena:
    
    def __init__(self) -> None:
        self._system = System()

        self._loop: asyncio.AbstractEventLoop | None = None
        self._executor = ThreadPoolExecutor(
            max_workers=self._system.configuration.cores.physical
        )

        self._client = None

        self.service_name = 'Athena'
        self._regions = AWSRegionMap()
    
    async def sso(
        self,
        profile_name: str,
    ):

        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        await self._loop.run_in_executor(
            None,
            functools.partial(
                boto3.setup_default_session,
                profile_name=profile_name
            )
        )

        self._client: AthenaClient = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                boto3.client,
                'athena',
            )
        )

    async def connect(
        self,
        credentials: AWSCredentialsSet,
        region: RegionName,
    ):
        
        aws_region = self._regions.get(region)

        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        self._client: AthenaClient = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                boto3.client,
                'athena',
                aws_access_key_id=credentials.aws_access_key_id,
                aws_secret_access_key=credentials.aws_secret_access_key,
                aws_session_token=credentials.aws_session_token,
                config=Config(
                    region_name=aws_region.value
                )
            )
        )

    async def create_data_catalog(
        self,
        request: AthenaCreateDataCatalogRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)

        response = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.create_data_catalog,
                **dumped
            )
        )

        return AthenaCreateDataCatalogResponse(**response)
    

    async def create_named_query(
        self,
        request: AthenaCreateNamedQueryRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)

        response = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.create_named_query,
                **dumped
            )
        )

        return AthenaCreateNamedQueryResponse(**response)
    
    async def create_prepared_statement(
        self,
        request: AthenaCreatePreparedStatementRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)

        response = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.create_prepared_statement,
                **dumped
            )
        )

        return AthenaCreatePreparedStatementResponse(**response)
    
    async def get_calculation_execution_code(
        self,
        request: AthenaGetCalculationExectutionCodeRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)

        response = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.get_calculation_execution_code,
                **dumped
            )
        )

        return AthenaGetCalculationExecutionCodeResponse(**response)

    async def get_calculation_execution(
        self,
        request: AthenaGetCalculationExecutionRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)

        response = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.get_calculation_execution,
                **dumped
            )
        )

        return AthenaGetCalculationExecutionResponse(**response)

    async def get_calculation_execution_status(
        self,
        request: AthenaGetCalculationExecutionStatusRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)

        response = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.get_calculation_execution_status,
                **dumped
            )
        )

        return AthenaGetCalculationExecutionStatusResponse(**response)

    async def get_data_catalog(
        self,
        request: AthenaGetDataCatalogRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)

        response = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.get_data_catalog,
                **dumped
            )
        )

        return AthenaGetDataCatalogResponse(**response)
    
    async def get_database(
        self,
        request: AthenaGetDatabaseRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)

        response = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.get_database,
                **dumped
            )
        )

        return AthenaGetDatabaseResponse(**response)
    
    async def get_named_query(
        self,
        request: AthenaGetNamedQueryRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)

        response = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.get_named_query,
                **dumped
            )
        )

        return AthenaGetNamedQueryResponse(**response)
    
    async def get_prepared_statement(
        self,
        request: AthenaGetPreparedStatementRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)

        response = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.get_prepared_statement,
                **dumped
            )
        )

        return AthenaGetPreparedStatementResponse(**response)

    async def get_query_execution(
        self,
        request: AthenaGetQueryExecutionRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)

        response = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.get_query_execution,
                **dumped
            )
        )

        return AthenaGetQueryExecutionResponse(**response)

    async def get_query_results(
        self,
        request: AthenaGetQueryResultsRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)

        response = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.get_query_results,
                **dumped
            )
        )

        return AthenaGetQueryResultsResponse(**response)

    async def get_session_request(
        self,
        request: AthenaGetSessionRequest,
    ):    
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)

        response = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.get_session,
                **dumped
            )
        )

        return AthenaGetSessionResponse(**response)
    
    async def get_session_status_request(
        self,
        request: AthenaGetSessionStatusRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)

        response = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.get_session_status,
                **dumped
            )
        )

        return AthenaGetSessionStatusResponse(**response)

    async def list_databases(
        self,
        request: AthenaListDatabasesRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)

        response = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.list_databases,
                **dumped
            )
        )

        return AthenaListDatabasesResponse(**response)

    async def list_databases_paginated(
        self,
        request: AthenaListDatabasesRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)

        paginator = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.get_paginator,
                'list_databases',
            )
        )

        pagination_result = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                paginator.paginate,
                **dumped
            )
        )

        results = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                collect_paginator,
                pagination_result,
            )
        )

        return [
            AthenaListDatabasesResponse(**res) for res in results
        ]
    
    async def list_data_catalogs(
        self,
        request: AthenaListDataCatalogsRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)

        response = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.list_data_catalogs,
                **dumped
            )
        )

        return AthenaListDataCatalogsResponse(**response)

    async def list_data_catalogs_paginated(
        self,
        request: AthenaListDataCatalogsRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)

        paginator = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.get_paginator,
                'list_data_catalogs',
            )
        )

        pagination_result = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                paginator.paginate,
                **dumped
            )
        )

        results = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                collect_paginator,
                pagination_result,
            )
        )

        return [
            AthenaListDataCatalogsResponse(**res) for res in results
        ]

    async def list_named_queries(
        self,
        request: AthenaListNamedQueriesRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)

        response = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.list_named_queries,
                **dumped
            )
        )

        return AthenaListNamedQueriesResponse(**response)
    
    async def list_named_queries_paginated(
        self,
        request: AthenaListNamedQueriesRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)

        paginator = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.get_paginator,
                'list_named_queries',
            )
        )

        pagination_result = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                paginator.paginate,
                **dumped
            )
        )

        results = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                collect_paginator,
                pagination_result,
            )
        )

        return [
            AthenaListNamedQueriesResponse(**res) for res in results
        ]

    async def list_prepared_statements(
        self,
        request: AthenaListPreparedStatementsRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)

        response = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.list_prepared_statements,
                **dumped
            )
        )

        return AthenaListPreparedStatementsResponse(**response)
    
    async def list_prepared_statements_paginated(
        self,
        request: AthenaListPreparedStatementsRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)

        paginator = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.get_paginator,
                'list_prepared_statements',
            )
        )

        pagination_result = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                paginator.paginate,
                **dumped
            )
        )

        results = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                collect_paginator,
                pagination_result,
            )
        )

        return [
            AthenaListPreparedStatementsResponse(**res) for res in results
        ]

    async def list_query_executions(
        self,
        request: AthenaListQueryExecutionsRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)

        response = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.list_query_executions,
                **dumped
            )
        )

        return AthenaListQueryExecutionsResponse(**response)
    
    async def list_query_executions_paginated(
        self,
        request: AthenaListQueryExecutionsRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)

        paginator = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.get_paginator,
                'list_query_executions',
            )
        )

        pagination_result = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                paginator.paginate,
                **dumped
            )
        )

        results = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                collect_paginator,
                pagination_result,
            )
        )

        return [
            AthenaListQueryExecutionsResponse(**res) for res in results
        ]
    
    async def list_sessions(
        self,
        request: AthenaListSessionsRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)

        response = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.list_sessions,
                **dumped
            )
        )

        return AthenaListSessionsResponse(**response)
        
    async def list_sessions_paginated(
        self,
        request: AthenaListSessionsRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)

        paginator = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.get_paginator,
                'list_sessions',
            )
        )

        pagination_result = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                paginator.paginate,
                **dumped
            )
        )

        results = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                collect_paginator,
                pagination_result,
            )
        )

        return [
            AthenaListSessionsResponse(**res) for res in results
        ]

    async def start_calculation_execution(
        self,
        request: AthenaStartCalculationExecutionRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)

        response = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.start_calculation_execution,
                **dumped
            )
        )

        return AthenaStartCalculationExecutionResponse(**response)  

    async def start_query_execution(
        self,
        request: AthenaStartQueryExecutionRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)

        response = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.start_query_execution,
                **dumped
            )
        )

        return AthenaStartQueryExecutionResponse(**response) 

    async def start_session(
        self,
        request: AthenaStartSessionRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)

        response = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.start_session,
                **dumped
            )
        )

        return AthenaStartSessionResponse(**response)   
    
    async def stop_calculation_execution(
        self,
        request: AthenaStopCalculationExecutionRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)
        
        response = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.stop_calculation_execution,
                **dumped
            )
        )

        return AthenaStopCalculationExecutionResponse(**response)
    
    async def stop_query_execution(
        self,
        request: AthenaStopQueryExecutionRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)
        
        response = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.stop_query_execution,
                **dumped
            )
        )

        return AthenaStopQueryExecutionResponse(**response)
    
    async def terminate_session(
        self,
        request: AthenaTerminateSessionRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)

        response = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.terminate_session,
                **dumped
            )
        )

        return AthenaTerminateSessionResponse(**response)
    
    async def update_data_catalog(
        self,
        request: AthenaUpdateDataCatalogRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)

        response = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.update_data_catalog,
                **dumped
            )
        )

        return AthenaUpdateDataCatalogResponse(**response)
    
    async def update_named_query(
        self,
        request: AthenaUpdateNamedQueryRequest,
    ):
        
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name
            )
        
        dumped = request.model_dump(exclude_none=True)

        response = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.update_named_query,
                **dumped
            )
        )

        return AthenaUpdateNamedQueryResponse(**response)
    
    async def close(self):

        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client:
            await self._loop.run_in_executor(
                None,
                self._client.close
            )

        await self._system.close()
        self._executor.shutdown()

    def abort(self):
        self._client.close()
        self._system.abort()
        self._executor.shutdown()


from pydantic import (
    BaseModel,
    StrictBool,
    StrictStr,
    StrictInt,
)


class AWSS3ListObjectsRequest(BaseModel):
    Bucket: StrictStr
    Delimiter: StrictStr | None = None
    EncodingType: StrictStr | None = None
    MaxKeys: StrictInt | None = None
    Prefix: StrictStr | None = None
    ContinuationToken: StrictStr | None = None
    FetchOwner: StrictBool | None = None
    StartAfter: StrictStr | None = None
    ExpectedBucketOwner: StrictStr | None = None
    OptionalObjectAttributes: list[StrictStr] | None = None


import datetime
from pydantic import (
    BaseModel,
    StrictStr,
)
from .athena_calculation_state import AthenaCalculationState

class AthenaCalculationStatus(BaseModel):
    SubmissionDateTime: datetime.datetime
    CompletionDateTime: datetime.datetime
    State: AthenaCalculationState
    StateChangeReason: StrictStr | None = None


from pydantic import (
    BaseModel,
    StrictStr,
)
from .athena_data_catalog_summary import AthenaDataCatalogSummary


class AthenaListDataCatalogsResponse(BaseModel):
    DataCatalogsSummary: list[AthenaDataCatalogSummary]
    NextToken: StrictStr | None = None


from typing import Literal

AWSS3StorageClass = Literal[
    'STANDARD',
    'REDUCED_REDUNDANCY',
    'STANDARD_IA',
    'ONEZONE_IA',
    'INTELLIGENT_TIERING',
    'GLACIER',
    'OUTPOSTS',
    'GLACIER_IR',
    'SNOW'
]


import datetime
from pydantic import (
    BaseModel,
    StrictStr,
)


class AthenaPreparedStatement(BaseModel):
    StatementName: StrictStr
    QueryStatement: StrictStr | None = None
    WorkGroupName: StrictStr | None = None
    Description: StrictStr | None = None
    LastModifiedTime: datetime.datetime


from pydantic import (
    BaseModel,
    StrictStr,
)


class AthenaCalculationResult(BaseModel):
    StdOutS3Uri: StrictStr
    StdErrorS3Uri: StrictStr
    ResultS3Uri: StrictStr
    ResultType: StrictStr


import datetime
from pydantic import BaseModel, StrictStr, StrictBool
from typing import Literal


class AWSs3CreateMultipartUploadRequest(BaseModel):
    Bucket: StrictStr
    Key: StrictStr
    ACL:  Literal[
        'private',
        'public-read',
        'public-read-write',
        'authenticated-read',
        'aws-exec-read',
        'bucket-owner-read',
        'bucket-owner-full-control'
    ] | None = None
    CacheControl: StrictStr | None = None
    ContentDisposition: StrictStr | None = None
    ContentEncoding: StrictStr | None = None
    ContentLanguage: StrictStr | None = None
    ContentType: StrictStr | None = None
    Expires: datetime.datetime | None = None
    GrantFullControl: StrictStr | None = None
    GrantRead: StrictStr | None = None
    GrantReadACP: StrictStr | None = None
    GrantWriteACP: StrictStr | None = None
    Metadata: dict[StrictStr, StrictStr] | None = None
    ServerSideEncryption: Literal[
        'AES256',
        'aws:kms',
        'aws:kms:dsse'
    ] | None = None
    StorageClass: Literal[
        'STANDARD',
        'REDUCED_REDUNDANCY',
        'STANDARD_IA',
        'ONEZONE_IA',
        'INTELLIGENT_TIERING',
        'GLACIER',
        'DEEP_ARCHIVE',
        'OUTPOSTS',
        'GLACIER_IR',
        'SNOW',
        'EXPRESS_ONEZONE'
    ] | None = None
    WebsiteRedirectLocation: StrictStr | None = None
    SSECustomerAlgorithm: StrictStr | None = None
    SSECustomerKey: StrictStr | None = None
    SSEKMSKeyId: StrictStr | None = None
    SSEKMSEncryptionContext: StrictStr | None = None
    BucketKeyEnabled: StrictBool | None = None
    RequestPayer: StrictStr | None = None
    Tagging: StrictStr | None = None
    ObjectLockMode: Literal[
        'GOVERNANCE',
        'COMPLIANCE'
    ] | None = None
    ObjectLockRetainUntilDate: datetime.datetime | None = None
    ObjectLockLegalHoldStatus: Literal[
        'ON',
        'OFF'
    ] | None = None
    ExpectedBucketOwner: StrictStr | None = None
    ChecksumAlgorithm: Literal[
        'CRC32',
        'CRC32C',
        'SHA1',
        'SHA256'
    ] | None = None
    ChecksumType: Literal["COMPOSITE", "FULL_OBJECT"] | None = None



from pydantic import BaseModel
from .athena_result_reuse_by_age_configuration import AthenaResultReuseByAgeConfiguration


class AthenaResultReuseConfiguration(BaseModel):
    ResultReuseByAgeConfiguration: AthenaResultReuseByAgeConfiguration


from .athena_acl_configuration import AthenaAclConfiguration as AthenaAclConfiguration
from .athena_calculation_configuration import AthenaCalculationConfiguration as AthenaCalculationConfiguration
from .athena_calculation_result import AthenaCalculationResult as AthenaCalculationResult
from .athena_calculation_state import AthenaCalculationState as AthenaCalculationState
from .athena_calculation_statistics import AthenaCalculationStatistics as AthenaCalculationStatistics
from .athena_calculation_status import AthenaCalculationStatus as AthenaCalculationStatus
from .athena_column_info import AthenaColumnInfo as AthenaColumnInfo
from .athena_column_nullable import AthenaColumnNullable as AthenaColumnNullable
from .athena_connection_type import AthenaConnectionType as AthenaConnectionType
from .athena_create_data_catalog_request import AthenaCreateDataCatalogRequest as AthenaCreateDataCatalogRequest
from .athena_create_data_catalog_response import AthenaCreateDataCatalogResponse as AthenaCreateDataCatalogResponse
from .athena_create_named_query_request import AthenaCreateNamedQueryRequest  as AthenaCreateNamedQueryRequest
from .athena_create_named_query_response import AthenaCreateNamedQueryResponse as AthenaCreateNamedQueryResponse
from .athena_create_prepared_statement_request import AthenaCreatePreparedStatementRequest as AthenaCreatePreparedStatementRequest
from .athena_create_prepared_statement_response import AthenaCreatePreparedStatementResponse as AthenaCreatePreparedStatementResponse
from .athena_data_catalog import AthenaDataCatalog as AthenaDataCatalog
from .athena_data_catalog_status import AthenaDataCatalogStatus as AthenaDataCatalogStatus
from .athena_data_catalog_summary import AthenaDataCatalogSummary as AthenaDataCatalogSummary
from .athena_data_catalog_type import AthenaDataCatalogType as AthenaDataCatalogType
from .athena_database import AthenaDatabase as AthenaDatabase
from .athena_delete_data_catalog_request import AthenaDeleteDataCatalogRequest as AthenaDeleteDataCatalogRequest
from .athena_delete_data_catalog_response import AthenaDeleteDataCatalogResponse as AthenaDeleteDataCatalogResponse
from .athena_delete_named_query_request import AthenaDeleteNamedQueryRequest as AthenaDeleteNamedQueryRequest
from .athena_delete_named_query_response import AthenaDeleteNamedQueryResponse as AthenaDeleteNamedQueryResponse
from .athena_delete_prepared_statement_request import AthenaDeletePreparedStatementRequest as AthenaDeletePreparedStatementRequest
from .athena_delete_prepared_statement_response import AthenaDeletePreparedStatementResponse as AthenaDeletePreparedStatementResponse
from .athena_encryption_configuration import AthenaEncryptionConfiguration as AthenaEncryptionConfiguration
from .athena_encryption_option import AthenaEncryptionOption as AthenaEncryptionOption
from .athena_engine_configuration import AthenaEngineConfiguration as AthenaEngineConfiguration
from .athena_engine_version import AthenaEngineVersion as AthenaEngineVersion
from .athena_get_calculation_execution_code_request import AthenaGetCalculationExectutionCodeRequest as AthenaGetCalculationExectutionCodeRequest
from .athena_get_calculation_execution_code_response import AthenaGetCalculationExecutionCodeResponse as AthenaGetCalculationExecutionCodeResponse
from .athena_get_calculation_execution_request import AthenaGetCalculationExecutionRequest as AthenaGetCalculationExecutionRequest
from .athena_get_calculation_execution_response import AthenaGetCalculationExecutionResponse as AthenaGetCalculationExecutionResponse
from .athena_get_calculation_execution_status_request import AthenaGetCalculationExecutionStatusRequest as AthenaGetCalculationExecutionStatusRequest
from .athena_get_calculation_execution_status_response import AthenaGetCalculationExecutionStatusResponse as AthenaGetCalculationExecutionStatusResponse
from .athena_get_data_catalog_request import AthenaGetDataCatalogRequest as AthenaGetDataCatalogRequest
from .athena_get_data_catalog_response import AthenaGetDataCatalogResponse as AthenaGetDataCatalogResponse
from .athena_get_database_request import AthenaGetDatabaseRequest as AthenaGetDatabaseRequest
from .athena_get_database_response import AthenaGetDatabaseResponse as AthenaGetDatabaseResponse
from .athena_get_named_query_request import AthenaGetNamedQueryRequest as AthenaGetNamedQueryRequest
from .athena_get_named_query_response import AthenaGetNamedQueryResponse as AthenaGetNamedQueryResponse
from .athena_get_prepared_statement_request import AthenaGetPreparedStatementRequest as AthenaGetPreparedStatementRequest
from .athena_get_prepared_statement_response import AthenaGetPreparedStatementResponse as AthenaGetPreparedStatementResponse
from .athena_get_query_execution_request import AthenaGetQueryExecutionRequest as AthenaGetQueryExecutionRequest
from .athena_get_query_execution_response import AthenaGetQueryExecutionResponse as AthenaGetQueryExecutionResponse
from .athena_get_query_results_request import AthenaGetQueryResultsRequest as AthenaGetQueryResultsRequest
from .athena_get_query_results_response import AthenaGetQueryResultsResponse as AthenaGetQueryResultsResponse
from .athena_get_session_request import AthenaGetSessionRequest as AthenaGetSessionRequest
from .athena_get_session_response import AthenaGetSessionResponse as AthenaGetSessionResponse
from .athena_get_session_status_request import AthenaGetSessionStatusRequest as AthenaGetSessionStatusRequest
from .athena_get_session_status_response import AthenaGetSessionStatusResponse as AthenaGetSessionStatusResponse
from .athena_list_data_catalogs_request import AthenaListDataCatalogsRequest as AthenaListDataCatalogsRequest
from .athena_list_data_catalogs_response import AthenaListDataCatalogsResponse as AthenaListDataCatalogsResponse
from .athena_list_databases_request import AthenaListDatabasesRequest as AthenaListDatabasesRequest
from .athena_list_databases_response import AthenaListDatabasesResponse as AthenaListDatabasesResponse
from .athena_list_named_queries_request import AthenaListNamedQueriesRequest as AthenaListNamedQueriesRequest
from .athena_list_named_queries_response import AthenaListNamedQueriesResponse as AthenaListNamedQueriesResponse
from .athena_list_prepared_statements_request import AthenaListPreparedStatementsRequest as AthenaListPreparedStatementsRequest
from .athena_list_prepared_statements_response import AthenaListPreparedStatementsResponse as AthenaListPreparedStatementsResponse
from .athena_list_query_executions_request import AthenaListQueryExecutionsRequest as AthenaListQueryExecutionsRequest
from .athena_list_query_executions_response import AthenaListQueryExecutionsResponse as AthenaListQueryExecutionsResponse
from .athena_list_sessions_request import AthenaListSessionsRequest as AthenaListSessionsRequest
from .athena_list_sessions_response import AthenaListSessionsResponse as AthenaListSessionsResponse
from .athena_managed_query_results_configuration import AthenaManagedQueryResultsConfiguration as AthenaManagedQueryResultsConfiguration
from .athena_named_query import AthenaNamedQuery as AthenaNamedQuery
from .athena_prepared_statement import AthenaPreparedStatement as AthenaPreparedStatement
from .athena_query_error import AthenaQueryError as AthenaQueryError
from .athena_query_execution_context import AthenaQueryExecutionContext as AthenaQueryExecutionContext
from .athena_query_execution import AthenaQueryExecution as AthenaQueryExecution
from .athena_query_result_type import AthenaQueryResultType as AthenaQueryResultType
from .athena_query_results_s3_access_grants_configruation import AthenaQueryResultsS3AccessGrantsConfiguration as AthenaQueryResultsS3AccessGrantsConfiguration
from .athena_query_state import AthenaQueryState as AthenaQueryState
from .athena_query_statistics import AthenaQueryStatistics as AthenaQueryStatistics
from .athena_query_status import AthenaQueryStatus as AthenaQueryStatus
from .athena_result_configuration import AthenaResultConfigruation as AthenaResultConfigruation
from .athena_result_reuse_by_age_configuration import AthenaResultReuseByAgeConfiguration as AthenaResultReuseByAgeConfiguration
from .athena_result_reuse_information import AthenaResultReuseInformation as AthenaResultReuseInformation
from .athena_result_set_metadata import AthenaResultSetMetadata as AthenaResultSetMetadata
from .athena_result_set import AthenaResultSet as AthenaResultSet
from .athena_row import AthenaRow as AthenaRow
from .athena_session_configuration import AthenaSessionConfiguration as AthenaSessionConfiguration
from .athena_session_state import AthenaSessionState as AthenaSessionState
from .athena_session_statistics import AthenaSessionStatistics as AthenaSessionStatistics
from .athena_session_status import AthenaSessionStatus as AthenaSessionStatus
from .athena_session import AthenaSession as AthenaSession
from .athena_start_calculation_execution_request import AthenaStartCalculationExecutionRequest as AthenaStartCalculationExecutionRequest
from .athena_start_calculation_execution_response import AthenaStartCalculationExecutionResponse as AthenaStartCalculationExecutionResponse
from .athena_start_query_execution_request import AthenaStartQueryExecutionRequest as AthenaStartQueryExecutionRequest
from .athena_start_query_execution_response import AthenaStartQueryExecutionResponse as AthenaStartQueryExecutionResponse
from .athena_start_session_request import AthenaStartSessionRequest as AthenaStartSessionRequest
from .athena_start_session_response import AthenaStartSessionResponse as AthenaStartSessionResponse
from .athena_statement_type import AthenaStatementType as AthenaStatementType
from .athena_stop_calculation_execution_request import AthenaStopCalculationExecutionRequest as AthenaStopCalculationExecutionRequest
from .athena_stop_calculation_execution_response import AthenaStopCalculationExecutionResponse as AthenaStopCalculationExecutionResponse
from .athena_stop_query_execution_request import AthenaStopQueryExecutionRequest as AthenaStopQueryExecutionRequest
from .athena_stop_query_execution_response import AthenaStopQueryExecutionResponse as AthenaStopQueryExecutionResponse
from .athena_tag import AthenaTag as AthenaTag
from .athena_terminate_session_request import AthenaTerminateSessionRequest as AthenaTerminateSessionRequest
from .athena_terminate_session_response import AthenaTerminateSessionResponse as AthenaTerminateSessionResponse
from .athena_update_data_catalog_request import AthenaUpdateDataCatalogRequest as AthenaUpdateDataCatalogRequest
from .athena_update_data_catalog_response import AthenaUpdateDataCatalogResponse as AthenaUpdateDataCatalogResponse
from .athena_update_named_query_request import AthenaUpdateNamedQueryRequest as AthenaUpdateNamedQueryRequest
from .athena_update_named_query_response import AthenaUpdateNamedQueryResponse as AthenaUpdateNamedQueryResponse
from .athena_var_char_value import AthenaVarCharValue as AthenaVarCharValue


from pydantic import (
    BaseModel,
    StrictInt,
    StrictBool,
    StrictStr,
)

class AthenaQueryError(BaseModel):
    ErrorCategory: StrictInt
    ErrorType: StrictInt
    Retryable: StrictBool = False
    ErrorMessage: StrictStr


from pydantic import BaseModel, StrictStr
from typing import Literal


class AWSs3AbortMultipartUploadRequest(BaseModel):
    Bucket: StrictStr
    Key: StrictStr
    UploadId: StrictStr
    RequestPayer: Literal['requester'] | None = None
    ExpectedBucketOwner: StrictStr | None=None


from pydantic import (
    StrictStr,
    StrictInt
)
from werkflow_aws.models.base import AWSBoto3Base


class AWSs3ListPartsRequest(AWSBoto3Base):
    Bucket: StrictStr
    Key: StrictStr
    MaxParts: StrictInt | None=None
    PartNumberMarker: StrictInt | None=None
    UploadId: StrictStr
    RequestPayer: StrictStr | None=None
    ExpectedBucketOwner: StrictStr | None=None
    SSECustomerAlgorithm: StrictStr | None=None
    SSECustomerKey: StrictStr | None=None



from pydantic import (
    BaseModel,
    StrictInt,
    StrictStr
)


class AthenaEngineConfiguration(BaseModel):
    CoordinatorDpuSize: StrictInt
    MaxConcurrentDpus: StrictInt
    DefaultExecutorDpuSize: StrictInt
    AdditionalConfigs: dict[StrictStr, StrictStr]
    SparkProperties: dict[StrictStr, StrictStr]